* ---------------------------------------------- ab06_fmrvs_evaluate_gmpes
      subroutine ab06_fmrvs_evaluate_gmpes(freq, m, rcd, v30, stress, 
     :                        stress_adj_factor_delta, 
     :                        stress_adj_factor_ml, 
     :                        stress_adj_factor_mh,
     :                        y, sigy, 
     :                        freq_tbl, coeff, nfreq,
     :                    amp_lin, amp_nonlin, amp_total)

* Computes ab06 motions.  Units are cgs

* Needs subroutines:
*   get_lun.for
*   skip.for
*   upstr.for
*   lin_interp.for
*   locate.for

* Dates: 11/26/05 - Written by D.M. Boore
*        11/12/06 - Pass coefficient array as arguments, rather than
*                   opening the coefficient table and reading the
*                   coefficients for each call of this subroutine.
*                   Because of this, vs is no longer needed as an argument.
*        02/05/10 - Do not interpolate coefficients.   Rather, find the
*                   index corresponding to the input frequency (which means
*                   that the calling program must use a valid frequency
*                   in the subroutine call).
*        02/06/10 - Add site amplifications
 
      real freq, m, rcd, alogy, sigy
      real freq_tbl(*), coeff(50,10)
      real stress_adj_factor_delta(*), 
     :     stress_adj_factor_ml(*), 
     :     stress_adj_factor_mh(*)
      real delta, ml, mh
      
      character nonlin_corr*1
   
 
     
* Find index in table for input frequency:

      IF (freq .lt. 0.0 .or. freq .eq. 99.0) THEN
        IF (freq .lt. 0.0) THEN      ! pgv
          j = nfreq  
        ELSE                         ! pga
          j = nfreq - 1
        END IF
      ELSE
        j = 0
        find_index: DO i = 1, nfreq-2
          IF (freq == freq_tbl(i)) THEN
            j = i
            EXIT find_index
          END IF
        END DO find_index
      END IF
      
      IF (j == 0) THEN
        write(*,*) 
     :      ' In ab06_sub: Index for coefficient cannot be'//
     :      ' found for frequency ', freq,' QUITTING'
        STOP
      END IF
      
      c1_i = coeff(j,1)
      c2_i = coeff(j,2)
      c3_i = coeff(j,3)
      c4_i = coeff(j,4)
      c5_i = coeff(j,5)
      c6_i = coeff(j,6)
      c7_i = coeff(j,7)
      c8_i = coeff(j,8)
      c9_i = coeff(j,9)
      c10_i = coeff(j,10)
 
      IF (rcd .eq. 1.0) THEN      
        write(*,'(a)') 
     :     ' freq, j, freq_tbl(j), c1_i, c10_i = ' 
        write(*,*) 
     :       freq, j, freq_tbl(j), c1_i, c10_i          
      END IF


 
      r0 = 10.0
      r1 = 70.0
      r2 = 140.0
      f0 = amax1(alog10(r0/rcd),0.0)
      f1 = amin1(alog10(rcd), alog10(r1))
      f2 = amax1(alog10(rcd/r2),0.0)
      
      s = 0.0
      
      alogy = c1_i + c2_i*m + c3_i*m**2
     :     + (c4_i + c5_i*m)*f1
     :     + (c6_i + c7_i*m)*f2
     :     + (c8_i + c9_i*m)*f0
     :     +  c10_i*rcd + s

* Include stress adjustment factor:

      delta = stress_adj_factor_delta(j)
      ml = stress_adj_factor_ml(j)
      mh = stress_adj_factor_mh(j)

      term1 = 0.05 + delta
      term2 = 0.05 + delta * amax1((m-ml),0.0)/(mh-ml) 
      
      alogsf2 = amin1(term1, term2)
      
      alogy = alogy + (alog10(stress/140.0)/alog10(2.0))*alogsf2
      y_no_site_amp = 10.0**alogy

* Site amplification:

      if (v30 == 760.0 .or. v30 >= 2000.0) then
        amp_total = 1.0
        amp_lin = 1.0
        amp_nonlin = 1.0
      else
* Compute pgaref:
        jpga = nfreq - 1
        c1_pga  = coeff(jpga,1)
        c2_pga  = coeff(jpga,2)
        c3_pga  = coeff(jpga,3)
        c4_pga  = coeff(jpga,4)
        c5_pga  = coeff(jpga,5)
        c6_pga  = coeff(jpga,6)
        c7_pga  = coeff(jpga,7)
        c8_pga  = coeff(jpga,8)
        c9_pga  = coeff(jpga,9)
        c10_pga = coeff(jpga,10)

        s = 0.0
      
        alogpgaref = c1_pga + c2_pga*m + c3_pga*m**2
     :     + (c4_pga + c5_pga*m)*f1
     :     + (c6_pga + c7_pga*m)*f2
     :     + (c8_pga + c9_pga*m)*f0
     :     +  c10_pga*rcd + s
     
        pgaref = 10.0**alogpgaref
        pgaref_g = pgaref/981.0
        
        a1 = 0.03
        pga_g_low = 0.06
        a2 = 0.09
        v30ref = 760.0
        imodel_lin = 2
        imodel_nl = 3
        nonlin_corr = 'Y'
        
        if (freq == -1.0) then
          per = 0.5
        else if (freq == 99.0) then
          per = 0.0
        else
          per = 1.0/freq
        end if
        
        call site_amp_correction(
     :      per, v30, v30ref, imodel_lin, 
     :      bv, va,
     :      nonlin_corr, imodel_nl, pgaref_g,
     :      a1, pga_g_low, a2,
     :      bnl, amp_lin, amp_nonlin, amp_total)
      
*DEBUG
!        write(*,*) ' In ab06_fmrvs_evaluate_gmpes: '//
!     :  'per, pgaref_g, amp_lin, amp_nonlin, amp_total = '
!        write(*,*)  
!     :   per, pgaref_g, amp_lin, amp_nonlin, amp_total
*DEBUG


      end if
      y = amp_total * y_no_site_amp

*DEBUG
!        if (freq == -1.0) then
!          per = 0.5
!        else if (freq == 99.0) then
!          per = 0.0
!        else
!          per = 1.0/freq
!        end if
!        
!      write(*,*) ' In ab06_fmrvs_evaluate_gmpes: '//
!     :  'per, y_no_site_amp, y, y_g = ', 
!     :   per, y_no_site_amp, y, y/981.0
*DEBUG

* Assume a value for sigma:

      slogy = 0.30
      sigY = 10.0**slogY

      RETURN
      END
* ---------------------------------------------------- ab06_fmrvs_evaluate_gmpes

 
