
! --------------------------- BEGIN GET_LUN ----------------
      subroutine get_lun(lun)

* Finds a logical unit number not in use; returns
* -1 if it cannot find one.

* Dates -- 05/19/98 - Written by D. Boore, following
*                     Larry Baker's suggestion

      logical isopen
      do i = 99,10,-1
        inquire (unit=i, opened=isopen)
        if(.not.isopen) then
          lun = i
          return
        end if
      end do
      lun = -1

      return
      end
! --------------------------- END GET_LUN ----------------
     
! ------------------------------------------------------------------ skipcmnt
      subroutine skipcmnt(nu, comment, ncomments)

* Skip text comments in the file attached to unit nu, but save skipped 
* comments in character array comment.  Skip at least one line, and more as 
* long as the lines are preceded by "|" or "!".

* Dates: 04/16/01 - Written by D. Boore
*        12/07/01 - Added check for eof
*        11/04/03 - Use trim_c to trim possible leading blank
*        02/03/07 - Initialize comments to blank

      character comment(*)*(*), buf*80

      ncomments = 0
100   buf = ' '
      read (nu,'(a)',end=999) buf
      call trim_c(buf,nc_buf)
      if (buf(1:1) .eq.'!' .or. buf(1:1) .eq.'|' .or. 
     :                     ncomments + 1 .eq. 1) then
        ncomments = ncomments + 1
        comment(ncomments) = ' '
        comment(ncomments) = buf(1:nc_buf)
        goto 100
      else 
        backspace nu
      end if

999   continue
 
      return
      end
! ------------------------------------------------------------------ skipcmnt
! ---------------------- BEGIN SKIP -------------------
      subroutine SKIP(lunit, nlines)
        if (nlines .lt. 1) then
          return
        else
          do i = 1, nlines
             read(lunit, *)
          end do
          return
        end if
      end
! ---------------------- END SKIP -------------------
! --------------------- BEGIN UPSTR ----------------------------------
      Subroutine UPSTR ( text )
* Converts character string in TEXT to uppercase
* Dates: 03/12/96 - Written by Larry Baker

C
      Implicit   None
C
      Character  text*(*)
C
      Integer    j
      Character  ch
C
      Do 1000 j = 1,LEN(text)
         ch = text(j:j)
         If ( LGE(ch,'a') .and. LLE(ch,'z') ) Then
            text(j:j) = CHAR ( ICHAR(ch) - ICHAR('a') + ICHAR('A') )
         End If
 1000    Continue
C
      Return
      End
! --------------------- END UPSTR ----------------------------------
! --------------------------- BEGIN TRIM_C -----------------------
      subroutine trim_c(cstr, nchar)

* strips leading and trailing blanks from cstr, returning the
* result in cstr, which is now nchar characters in length

* Strip off tabs also.

* Here is a sample use in constructing a column header, filled out with 
* periods:

** Read idtag:
*        idtag = ' '
*        read(nu_in, '(1x,a)') idtag
*        call trim_c(idtag, nc_id)
** Set up the column headings:
*        colhead = ' '
*        colhead = idtag(1:nc_id)//'......' ! nc_id + 6 > length of colhead

* Dates: 12/23/97 - written by D. Boore
*        12/08/00 - pad with trailing blanks.  Otherwise some comparisons
*                   of the trimmed character string with other strings
*                   can be in error because the original characters are left
*                   behind after shifting.  For example, here is a string
*                   before and after shifting, using the old version:
*                      col:12345
*                           MTWH  before
*                          MTWHH  after (but nc = 4).
*        03/21/01 - Check for a zero length input string
*        11/09/01 - Change check for zero length string to check for all blanks
*        10/19/09 - Strip off tabs

      character cstr*(*)

      if(cstr .eq. ' ') then
        nchar = 0
        return
      end if

      nend = len(cstr)

! Replace tabs with blanks:

      do i = 1, nend
        if(ichar(cstr(i:i)) .eq. 9) then
           cstr(i:i) = ' '
        end if
      end do



*      if(nend .eq. 0) then
*        nchar = 0
*        return
*      end if

      do i = nend, 1, -1
        if (cstr(i:i) .ne. ' ') then
           nchar2 = i
           goto 10
        end if
      end do

10    continue

      do j = 1, nchar2
        if (cstr(j:j) .ne. ' ') then
          nchar1 = j
          goto 20
        end if
      end do

20    continue
   
      nchar = nchar2 - nchar1 + 1
      cstr(1:nchar) = cstr(nchar1: nchar2)
      if (nchar .lt. nend) then
        do i = nchar+1, nend
          cstr(i:i) = ' '
        end do
      end if

      return
      end
! --------------------------- END TRIM_C -----------------------
! begin subroutine interpolate
      subroutine interpolate(y1, y2, x1, x2, x, y)
      
* Dates: 01/12/10 - Written by D. Boore

      real y1, y2, x1, x2, x, y, slope

      slope = (y2 - y1)/(x2-x1)
      y  = y1 + slope*(x - x1)
      
      return
      end
! end subroutine interpolate
      
      subroutine lin_interp(x, y, n, j, x_intrp, y_intrp)
      
* Computes linearly interpolated value of y

* Values out of range are assigned end values

* Dates: 03/16/05 - Written by D. Boore
*        07/24/05 - Added index j for end cases

      real x(*), y(*), slope, x_intrp, y_intrp
      integer j, n
      
      if (x_intrp .le. x(1)) then
        j = 1
        y_intrp = y(1)
        return
      end if
      
      if (x_intrp .ge. x(n)) then
        j = n
        y_intrp = y(n)
        return
      end if          
      
      call locate(x,n,x_intrp,j)
      
      slope = (y(j+1) - y(j))/(x(j+1)-x(j))
      y_intrp = y(j) + slope*(x_intrp - x(j))
      
      return
      end
      


!* --------------------- BEGIN LOCATE -----------------
      SUBROUTINE locate(xx,n,x,j)
      
* Comments added by D. Boore on 26feb2010:
*  finds j such that xx(j) < x <= xx(j+1)
*  EXCEPT if x = xx(1), then j = 1 (logically it would be 0 from
*  the above relation, but this is the same returned value of j
*  for a point out of range).
*  Also, if x = xx(n), j = n-1, which is OK
*  Note that j = 0 or j = n indicates that x is out of range.
*
* See the program test_locate.for to test this routine.

      INTEGER j,n
      REAL x,xx(n)
      INTEGER jl,jm,ju
      jl=0
      ju=n+1
10    if(ju-jl.gt.1)then
        jm=(ju+jl)/2
        if((xx(n).ge.xx(1)).eqv.(x.ge.xx(jm)))then
          jl=jm
        else
          ju=jm
        endif
      goto 10
      endif
      if(x.eq.xx(1))then
        j=1
      else if(x.eq.xx(n))then
        j=n-1
      else
        j=jl
      endif
      return
      END
!* --------------------- END LOCATE -----------------

! ------------------------------------------------  site_amp_correction
      subroutine site_amp_correction(
     :      per, v30, v30ref, imodel_lin, 
     :      bv, va,
     :      nonlin_corr, imodel_nl, pga_g,
     :      a1, pga_g_low, a2,
     :      bnl, amp_lin, amp_nonlin, amp_total)
      
* Input parameters: 
*    per: period for site amp calculation 
*    v30: Vs30 for site amp calculation 
*    v30ref: Vs30 reference value (760 m/s used for BA08)
*    imodel_lin: specify which linear model to use (1=BJF, 2=CS05; 2 used for BA08)
*    nonlin_corr = 'Y' for nonlinear site amp calculation
*    imodel_nl: specify which nonlinear model to use (1=CS; 2=GMA; 3=DMB1; 4=DMB2; 3 used for BA08)
*    a1, pga_g_low, a2: parameters for transition from nonlinear to linear
*                       amplification for small values of pga_g: 
*                       a1 = 0.03, pga_g_low = 0.06, a2 = 0.09g used for BA08 NGA GMPEs.

* Output parameters: bv (in terms of common log), va, amp_lin
* Output parameters: bnl (in terms of common log), amp_nonlin, amp_total (=amp_lin*amp_nonlin)
*                    The amps are for Y, not log Y.

* Use site_amp_from_v30_vs_per.for to compute the site amp for a
* specified model, pga4nl, and Vs30 vs period.

* Dates: 08/02/05 - Written by D. Boore
*        07/03/06 - Add parameters a1, a2 for cubic smoothing
*        08/05/09 - Edited some of the comments to make clearer the
*                   input parameters used in the BA NGA GMPEs.
*        02/26/10 - Improved subroutines to interpolate coefficients.

      character nonlin_corr*1
 
      call site_amp_linear(per, v30, v30ref, imodel_lin, 
     :                            bv, va, amp_lin)
     
      if (nonlin_corr .eq. 'Y') then
        call site_amp_nonlinear(
     :    per, v30, v30ref, pga_g, imodel_nl,
     :    a1, pga_g_low, a2,
     :    bnl, amp_nonlin)
      else
        bnl = 0.0
        amp_nonlin = 1.0
      end if
      
      amp_total = amp_lin * amp_nonlin
      
      return
      
      end
! ------------------------------------------------  site_amp_correction
      
! ----------------------------------- BEGIN Site_Amp_Linear -------------------
      subroutine Site_Amp_Linear(per, v30, v30ref, imodel_lin, 
     :                            bv, va, amp_lin)
     
* Input parameters: per, v30, v30ref, imodel_lin
* Output parameters: bv (in terms of common log), va, amp_lin

* Returns the linear amplification, following the amp factor
* of BJF94, but using either the BJF slope (imodel = 1) or the 
* Choi and Stewart (2005) (imodel = 2) slope.  For BJF94, uses the cubic
* equations in Table 8 of BJF94 (for 5% damping and random component). 
* The value for per = 0.1 is used for all smaller periods and the per=2.0, 5.0 value 
* is used for periods longer than 2.0, 5.0 sec for imodel = 1, 2.  Note that this assumption is
* conservative; we expect the amplifications to reach unity for long enough
* periods (at least for Fourier spectra, what about for response spectra?)

* PGA corresponds to PER = 0.0

* PGV: If per < 0.0, use per = 0.5 to evaluate the amplification, following
* Bommer and Alarcon (Journal of Earthquake Engineering, Vol. 10, No. 1 (2006) 1-31).
* But I am not convinced that their suggestion is always the best to use.  One way of
* dealing with this is to add a variable per4pgv to the input parameters.  The
* other way is to change the programs (and control files) that call this subprogram.  
* This is probably the best way of dealing with this, as it only requires a change in
* the calling program (and control files). Adding a variable per4pgv would require
* changes in this subprogram and in the calling program (and control files).
 
* Dates: 08/01/05 - Written by D. Boore, based on BJF94V30amp.FOR
*        03/17/07 - Added values for T = 7.5, 10.0 sec.
*        03/27/07 - I corrected an error (the if statement)
*        01/10/10 - Added note above about period for pgv
*        02/26/10 - Interpolate coefficients.
 
      real per, v30, bv, va, amp
      real cbv(0:3), cva(0:3)
      real log_per_cs05(30), per_cs05(30), c_cs05(30)
      real log_t
      
      data per_cs05/0.010,0.020,0.030,0.040,0.050,
     :              0.060,0.075,0.090,0.100,0.120,
     :              0.150,0.170,0.200,0.240,0.300,
     :              0.360,0.400,0.460,0.500,0.600,
     :              0.750,0.850,1.000,1.500,2.000,
     :              3.000,4.000,5.000,
     :              7.500,10.00/
     
      data log_per_cs05/-2.00000,-1.69897,-1.52288,-1.39794,-1.30103,
     :                  -1.22185,-1.12494,-1.04576,-1.00000,-0.92082,
     :                  -0.82391,-0.76955,-0.69897,-0.61979,-0.52288,
     :                  -0.44370,-0.39794,-0.33724,-0.30103,-0.22185,
     :                  -0.12494,-0.07058, 0.00000, 0.17609, 0.30103,
     :                   0.47712, 0.60206, 0.69897, 0.87506, 1.00000/

      data   c_cs05/-0.36,-0.34,-0.33,-0.31,-0.29,
     :              -0.25,-0.23,-0.23,-0.25,-0.26,
     :              -0.28,-0.29,-0.31,-0.38,-0.44,
     :              -0.48,-0.50,-0.55,-0.60,-0.66,
     :              -0.69,-0.69,-0.70,-0.72,-0.73,
     :              -0.74,-0.75,-0.75,
     :              -0.692, -0.650/
     
      nper_cs05 = 30
       
      cbv(0) = -0.21172
      cbv(1) =  0.06619
      cbv(2) = -1.35085
      cbv(3) =  0.79809
      nc_bv = 4
      
      cva(0) =  3.04586
      cva(1) =  1.69975
      cva(2) = -2.97445
      cva(3) =  1.37668
      nc_va = 4
 
      if (imodel_lin .eq. 1) then  ! BJF94 model
      
        if(per .eq. 0.0) then
          bv = -0.371
          va = 1396.0
        else 
          if (per .lt. 0.0) then
            t = 0.5
          else if (per .gt. 0.0 .and. per .lt. 0.1) then
            t = 0.1
          else if (per .gt. 2.0) then
            t = 2.0
          else
            t = per
          end if
          
          bv = cubic(t, cbv, nc_bv)
          va = 10.0**cubic(t, cva, nc_va) 
          
        end if

        ampv30 =    10.0**(bv*alog10(v30/va))
        ampv30ref = 10.0**(bv*alog10(v30ref/va))
        
        amp_lin = ampv30/ampv30ref
        
      else ! CS05 model
 
        if(per .eq. 0.0) then
          t = 0.01
        else 
          if (per .lt. 0.0) then
            t = 0.5
          else if (per .gt. 10.0) then ! corrected on 27 March 2007
                                       ! previous statement used 5.0 rather than 10.0
            t = 10.0
          else
            t = per
          end if          
        end if
        
        log_t = alog10(t)
        
        call lin_interp(log_per_cs05, c_cs05, nper_cs05, 
     :                          j, log_t, bv)
     
        va = v30ref
        amp_lin  =  10.0**(bv*alog10(v30/v30ref))
        
      end if
      
      return
      end

      function cubic(t, c, nc)
      
      real t, x, cubic
      integer nc, i
      real c(0:*)
      
      x = alog10(t/0.1)
      
      cubic = 0.0
      
      do i = 0, nc-1
        cubic = cubic + c(i)*x**float(i)
      end do
      
      return
      end
! ----------------------------------- END Site_Amp_Linear -------------------
  
           
! ----------------------------------- BEGIN site_amp_nonlinear -------------------  
      subroutine site_amp_nonlinear(
     :    per, v30, v30ref, pga_g, imodel_nl,
     :    a1, pga_g_low, a2,
     :    bnl, amp_nl)
      
* Input parameters: per, v30, v30ref, pga_g, imodel_nl, a1, pga_g_low, a2
* Output parameters: bnl (in terms of common log), amp_nl

* Compute the nonlinear site amp.
*       imodel_nl = 3 ! 1=CS; 2=GMA; 3=DMB1; 4=DMB2
* Used in BA08 NGA GMPEs: v30ref=760.0, imodel_nl = 3, a1 = 0.03, pga_g_low = 0.06, a2 = 0.09g

* NOTE: pga in g!!!!!

* Dates: 08/01/05 - Written by D. Boore
*        07/03/06 - Add parameters a1, a2 for cubic smoothing
*        08/05/09 - Edited some of the comments to make clearer the
*                   input parameters used in the BA NGA GMPEs.

             
      call nonlin_b(imodel_nl, per, v30, 
     :    bnl, b1_per, b2_per, v1_per, v2_per)
             
      if (a1 .eq. a2) then ! no smoothing
        if (pga_g .ge. pga_g_low) then
          amp_nl = (pga_g/0.10)**bnl
        else
          amp_nl = (pga_g_low/0.10)**bnl
        end if
      else ! smoothing
        dx = alog(a2/a1)
        dy = bnl*alog(a2/pga_g_low)
        c =  (3.0*dy-bnl*dx)/dx**2
        d = -(2.0*dy-bnl*dx)/dx**3
        if (pga_g .le. a1) then
          amp_nl = (pga_g_low/0.10)**bnl
        else if (pga_g .gt. a1 .and. pga_g .le. a2) then
          amp_nl = exp( bnl*alog(pga_g_low/0.1) +
     :                    c*(alog(pga_g/a1))**2 +
     :                    d*(alog(pga_g/a1))**3 )
        else
          amp_nl = (pga_g/0.10)**bnl
        end if
      end if    

      return
      
      end
! ----------------------------------- END site_amp_nonlinear -------------------  
      
! -------------------------------------------------------- nonlin_b
      subroutine nonlin_b(imodel, per, v30, 
     :   bnl, b1_per, b2_per, v1_per, v2_per)
     
* Input parameters: imodel, per, v30     
      
* PGA corresponds to PER = 0.0

* PGV: If per < 0.0, use per = 0.5 to evaluate the amplification, following
* Bommer and Alarcon (Journal of Earthquake Engineering, Vol. 10, No. 1 (2006) 1-31).
* But I am not convinced that their suggestion is always the best to use.  One way of
* dealing with this is to add a variable per4pgv to the input parameters.  The
* other way is to change the programs (and control files) that call this subprogram.  
* This is probably the best way of dealing with this, as it only requires a change in
* the calling program (and control files). Adding a variable per4pgv would require
* changes in this subprogram and in the calling program (and control files).

* Dates: 08/10/05 - Corrected error for pgv (per < 0.0)  
*        03/18/07 - Added values for T = 5.0, 7.5, 10.0 sec.
*        01/10/10 - Added note above about period for pgv
*        02/26/10 - Interpolate coefficients.
!        03/23/10 - Correct error setting per_interp = 0.0 for pga
!                   (error because take log (per_interp)

      real  Tcs(30), log_Tcs(30), b1(30), b2(30)
      real log_per_interp
 	
      data Tcs/0.010, 0.020, 0.030, 0.040, 0.050,
     :         0.060, 0.075, 0.090, 0.100, 0.120,
     :         0.150, 0.170, 0.200, 0.240, 0.300,
     :         0.360, 0.400, 0.460, 0.500, 0.600,
     :         0.750, 0.850, 1.000, 1.500, 2.000,
     :         3.000, 4.000, 5.000,
     :         7.500, 10.00/
     
      data log_Tcs/-2.00000,-1.69897,-1.52288,-1.39794,-1.30103,
     :             -1.22185,-1.12494,-1.04576,-1.00000,-0.92082,
     :             -0.82391,-0.76955,-0.69897,-0.61979,-0.52288,
     :             -0.44370,-0.39794,-0.33724,-0.30103,-0.22185,
     :             -0.12494,-0.07058, 0.00000, 0.17609, 0.30103,
     :              0.47712, 0.60206, 0.69897, 0.87506, 1.00000/

      data b1/-0.64, -0.63, -0.62, -0.61, -0.64,
     :        -0.64, -0.64, -0.64, -0.60, -0.56,
     :        -0.53, -0.53, -0.52, -0.52, -0.52,
     :        -0.51, -0.51, -0.50, -0.50, -0.49,
     :        -0.47, -0.46, -0.44, -0.40, -0.38,
     :        -0.34, -0.31, -0.291, 
     :        -0.247,-0.215/                                  
! replace old value of -0.30 for T = 5 
 
      data b2/-0.14, -0.12, -0.11, -0.11, -0.11,
     :        -0.11, -0.11, -0.12, -0.13, -0.14,
     :        -0.18, -0.19, -0.19, -0.16, -0.14,
     :        -0.11, -0.10, -0.08, -0.06, -0.03,
     :        -0.00, -0.00, -0.00, -0.00, -0.00,
     :        -0.00, -0.00, -0.00,
     :        -0.00, -0.00/
     
      n_cs = 30

      if (per .lt. 0.0) then  ! pgv
        per_interp = 0.5
      else if (per .eq. 0.0) then ! pga
        per_interp = 0.01   ! assume b1, b2 for pga correspond to Tcs=0.01
      else
        per_interp = per
      end if
      
      log_per_interp = alog10(per_interp)
      
      bv = 300.0 ! Choi & Stewart, Table 3, not same as BJF bv


      call lin_interp(log_Tcs, b1,  n_cs, ics1, log_per_interp, b1_per)
      call lin_interp(log_Tcs, b2,  n_cs, ics2, log_per_interp, b2_per)
      
      if (imodel .eq. 1) then ! Choi and Stewart model

        v1cs = 180.0
        v2cs = 520.0
        vacs = 760.0
      
        v1_per = v1cs
        v2_per = v2cs
        
         if (v30 .lt. v1cs) then
           bnl = b1_per
         else if (v30 .ge. v1cs .and. 
     :            v30 .lt. bv)  then
           bnl = b2_per + 
     :                (b1_per-b2_per)*((v30-bv)/(v1cs-bv))**2
         else if (v30 .ge. bv .and. 
     :            v30 .lt. v2cs)  then
           bnl = b2_per
         else if (v30 .ge. v2cs .and. 
     :            v30 .lt. vacs)  then
           bnl = b2_per - (v30-v2cs)*(b2_per/240.0)
         else
           bnl = 0.0
         end if

      else if (imodel .eq. 2) then ! Atkinson model

        v1gma = 170.0
        vagma = 760.0
        v1_per = v1gma
        v2_per = 0.0
       
         if (v30 .lt. vagma) then
           bnl = b1_per * 
     :          alog(v30/vagma)/alog(v1gma/vagma)
         else
           bnl = 0.0
         end if

      else if (imodel .eq. 3) then ! Dave's model 1; used for BA NGA equations

        v1dmb = 180.0
        v2dmb = bv
        vadmb = 760.0
        v1_per = v1dmb
        v2_per = v2dmb
      
         if (v30 .lt. v1dmb) then
           bnl = b1_per
         else if (v30 .ge. v1dmb .and. 
     :            v30 .lt. v2dmb)  then
           bnl = (b1_per-b2_per) * 
     :         alog(v30/v2dmb)/alog(v1dmb/v2dmb) + b2_per 
         else if (v30 .ge. v2dmb .and. 
     :            v30 .lt. vadmb)  then
           bnl = b2_per * 
     :         alog(v30/vadmb)/alog(v2dmb/vadmb)            
         else
           bnl = 0.0
         end if

      else if (imodel .eq. 4) then ! Dave's model 2
                                   ! (slope increases for small v30)

        v1dmb = 180.0
        v2dmb = bv
        vadmb = 760.0
        v1_per = v1dmb
        v2_per = v2dmb
      
         if (v30 .lt. v1dmb) then
           bnl = b1_per * 
     :          alog(v30/vadmb)/alog(v1dmb/vadmb)
         else if (v30 .ge. v1dmb .and. 
     :            v30 .lt. v2dmb)  then
           bnl = (b1_per-b2_per) * 
     :         alog(v30/v2dmb)/alog(v1dmb/v2dmb) + b2_per 
         else if (v30 .ge. v2dmb .and. 
     :            v30 .lt. vadmb)  then
           bnl = b2_per * 
     :         alog(v30/vadmb)/alog(v2dmb/vadmb)            
         else
           bnl = 0.0
         end if

      else
 
        print *,' imodel ',imodel,' not valid; quitting!'
        stop
 
      end if
      
      return
      end    
! -------------------------------------------------------- nonlin_b
  
      
       